import { useMemo } from 'react';
import { HSLDifference } from '../../color_functions/color_module';
import { useColorFunction } from '../../ColorFunctionContext';
import styles from './CompareLayout.module.css';
import { CompareMethodProps } from './share-props';

const defaultCompareResult: HSLDifference = {
  hue: {
    delta: 0,
    percent: 0,
  },
  saturation: {
    delta: 0,
    percent: 0,
  },
  lightness: {
    delta: 0,
    percent: 0,
  },
};

export function HSLCompare({
  basic = '000000',
  compare = '000000',
  mode = 'absolute',
}: CompareMethodProps) {
  const { colorFn } = useColorFunction();
  const differ = useMemo(() => {
    if (!colorFn) {
      return defaultCompareResult;
    }
    try {
      const diffFn = {
        absolute: colorFn.differ_in_hsl,
        relative: colorFn.relative_differ_in_hsl,
      };
      const diff = diffFn[mode](basic, compare);
      return diff;
    } catch (e) {
      console.error('[Compare in HSL]', e);
    }
    return defaultCompareResult;
  }, [basic, compare, mode]);

  return (
    <div>
      <h6>Compare in HSL Space</h6>
      <div className={styles.elements}>
        <div className={styles.element}>
          <div className={styles.element_name}>H</div>
          <div className={styles.element_values}>
            <span>{differ.hue.delta.toFixed(2)}</span>
            <span>
              {isNaN(differ.hue.percent) ? '0.00' : (differ.hue.percent * 100).toFixed(2)}%
            </span>
          </div>
        </div>
        <div className={styles.element}>
          <div className={styles.element_name}>S</div>
          <div className={styles.element_values}>
            <span>{(differ.saturation.delta * 100).toFixed(2)}%</span>
            <span>
              {isNaN(differ.saturation.percent)
                ? '0.00'
                : (differ.saturation.percent * 100).toFixed(2)}
              %
            </span>
          </div>
        </div>
        <div className={styles.element}>
          <div className={styles.element_name}>L</div>
          <div className={styles.element_values}>
            <span>{(differ.lightness.delta * 100).toFixed(2)}%</span>
            <span>
              {isNaN(differ.lightness.percent)
                ? '0.00'
                : (differ.lightness.percent * 100).toFixed(2)}
              %
            </span>
          </div>
        </div>
      </div>
    </div>
  );
}
