import { last } from 'lodash-es';
import { useMemo } from 'react';
import { useColorFunction } from '../../ColorFunctionContext';
import { FlexColorStand } from '../../components/FlexColorStand';

type ShadesListProps = {
  color: string;
  shades: number;
  mix: 'progressive' | 'linear' | 'average';
  step?: number;
  maximum?: number;
  copyMode: 'hex' | 'rgb' | 'hsl' | 'lab' | 'oklch';
};

export function Shades({ color, shades, mix, step, maximum, copyMode }: ShadesListProps) {
  const { colorFn } = useColorFunction();
  const colors = useMemo(() => {
    try {
      if (!colorFn) {
        return Array.from({ length: shades + 1 }, () => color);
      }
      const genColors = [color];
      switch (mix) {
        case 'progressive':
          for (let i = 1; i <= shades; i++) {
            const shade = colorFn!.shade(last(genColors) ?? '', step ?? 0);
            genColors.push(shade);
          }
          break;
        case 'linear':
          for (let i = 1; i <= shades; i++) {
            const shade = colorFn!.shade(color, (step ?? 0) * i);
            genColors.push(shade);
          }
          break;
        case 'average': {
          const interval = (maximum ?? 0) / shades / 100;
          for (let i = 1; i <= shades; i++) {
            const shade = colorFn!.shade(color, interval * i);
            genColors.push(shade);
          }
          break;
        }
      }
      return genColors.reverse();
    } catch (e) {
      console.error('[Generate Shades]', e);
    }
    return Array.from({ length: shades + 1 }, () => color);
  }, [color, shades, mix, step, maximum]);

  return (
    <>
      {colors.map((c, index) => (
        <FlexColorStand key={`${c}-${index}`} color={c} valueMode={copyMode} />
      ))}
    </>
  );
}
